@@ -10,12 +10,19 @@ use bevy_reflect::TypeUuid;
 #[uuid = "6378e9c2-ecd1-4029-9cd5-801caf68517c"]
 pub struct SpriteSheetAnimation {
     /// Frames
-    pub frames: Vec<Frame>,
+    pub(crate) frames: Vec<Frame>,
     /// Animation mode
-    pub mode: AnimationMode,
+    pub(crate) mode: Mode,
 }
 
-/// Animation mode (run once or repeat)
+/// Animation mode (run once, repeat or ping-pong)
+///
+/// Deprecated
+/// ---
+/// This is not exposed in any of the public APIs of the crate so there is no reason to depend on
+/// it. Use 'builder-style' methods like [`SpriteSheetAnimation::repeat`] instead.
+#[deprecated]
+#[non_exhaustive]
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum AnimationMode {
     /// Runs the animation once and then stop playing
@@ -23,15 +30,19 @@ pub enum AnimationMode {
 
     /// Repeat the animation forever
     Repeat,
+
+    /// Repeat the animation forever, going back and forth between
+    /// the first and last frame.
+    PingPong,
 }
 
 /// A single animation frame
 #[derive(Debug, Copy, Clone, Default)]
 pub struct Frame {
     /// Index in the sprite atlas
-    pub index: usize,
+    pub(crate) index: usize,
     /// How long should the frame be displayed
-    pub duration: Duration,
+    pub(crate) duration: Duration,
 }
 
 impl SpriteSheetAnimation {
@@ -40,7 +51,7 @@ impl SpriteSheetAnimation {
     pub fn from_frames(frames: Vec<Frame>) -> Self {
         Self {
             frames,
-            mode: AnimationMode::default(),
+            mode: Mode::default(),
         }
     }
 
@@ -56,15 +67,11 @@ impl SpriteSheetAnimation {
     ///
     /// # Example
     ///
+    /// You may use this to create a reversed animation:
     /// ```
-    /// # use benimator::{AnimationMode, SpriteSheetAnimation};
+    /// # use benimator::SpriteSheetAnimation;
     /// # use std::time::Duration;
-    /// // Easily create a reversed animation
     /// let animation = SpriteSheetAnimation::from_iter((0..5).rev(), Duration::from_millis(100));
-    ///
-    /// assert_eq!(animation.frames.iter().map(|frame| frame.index).collect::<Vec<_>>(), vec![4, 3, 2, 1, 0]);
-    /// assert!(animation.frames.iter().all(|frame| frame.duration.as_millis() == 100));
-    /// assert_eq!(animation.mode, AnimationMode::Repeat);
     /// ```
     ///
     /// For more granular configuration, see [`from_frames`](SpriteSheetAnimation::from_frames)
@@ -75,17 +82,32 @@ impl SpriteSheetAnimation {
             .collect()
     }
 
-    /// Set the animation mode to [`AnimationMode::Once`]
+    /// Runs the animation once and then stop playing
     #[must_use]
     pub fn once(mut self) -> Self {
-        self.mode = AnimationMode::Once;
+        self.mode = Mode::Once;
         self
     }
 
-    /// Set the animation mode to [`AnimationMode::Repeat`]
+    /// Repeat the animation forever
     #[must_use]
     pub fn repeat(mut self) -> Self {
-        self.mode = AnimationMode::Repeat;
+        self.mode = Mode::RepeatFrom(0);
+        self
+    }
+
+    /// Repeat the animation forever, from a given frame index (loop back to it at the end of the
+    /// animation)
+    #[must_use]
+    pub fn repeat_from(mut self, frame_index: usize) -> Self {
+        self.mode = Mode::RepeatFrom(frame_index);
+        self
+    }
+
+    /// Repeat the animation forever, going back and forth between the first and last frame.
+    #[must_use]
+    pub fn ping_pong(mut self) -> Self {
+        self.mode = Mode::PingPong;
         self
     }
 
@@ -94,12 +116,27 @@ impl SpriteSheetAnimation {
     }
 }
 
+#[derive(Debug, Copy, Clone, Eq, PartialEq)]
+pub(crate) enum Mode {
+    Once,
+    RepeatFrom(usize),
+    PingPong,
+}
+
 impl FromIterator<Frame> for SpriteSheetAnimation {
     fn from_iter<T: IntoIterator<Item = Frame>>(iter: T) -> Self {
         Self::from_frames(iter.into_iter().collect())
     }
 }
 
+impl Default for Mode {
+    #[inline]
+    fn default() -> Self {
+        Self::RepeatFrom(0)
+    }
+}
+
+#[allow(deprecated)]
 impl Default for AnimationMode {
     #[inline]
     fn default() -> Self {
