@@ -6,7 +6,7 @@ use bevy_core::prelude::*;
 use bevy_ecs::prelude::*;
 use bevy_sprite::prelude::*;
 
-use crate::{AnimationMode, Play, SpriteSheetAnimation};
+use crate::{animation::Mode, Play, SpriteSheetAnimation};
 
 pub(crate) fn maintenance_systems() -> SystemSet {
     SystemSet::new()
@@ -38,6 +38,8 @@ pub(crate) fn post_update_system() -> impl System<In = (), Out = ()> {
 pub struct SpriteSheetAnimationState {
     current_frame: usize,
     elapsed_in_frame: Duration,
+    // Control ping_pong backward frame navigation.
+    going_backward: bool,
 }
 
 impl SpriteSheetAnimationState {
@@ -63,13 +65,37 @@ impl SpriteSheetAnimationState {
 
         self.elapsed_in_frame += delta;
         if self.elapsed_in_frame >= frame.duration {
-            if self.current_frame < animation.frames.len() - 1 {
-                self.current_frame += 1;
-            } else if matches!(animation.mode, AnimationMode::Repeat) {
-                self.current_frame = 0;
-            } else {
-                self.reset();
-                return true;
+            match animation.mode {
+                Mode::RepeatFrom(loop_from) => {
+                    if self.current_frame < animation.frames.len() - 1 {
+                        self.current_frame += 1;
+                    } else {
+                        self.current_frame = loop_from;
+                    }
+                }
+                Mode::PingPong => {
+                    if self.going_backward {
+                        if self.current_frame == 0 {
+                            self.going_backward = false;
+                            self.current_frame += 1;
+                        } else {
+                            self.current_frame -= 1;
+                        }
+                    } else if self.current_frame < animation.frames.len() - 1 {
+                        self.current_frame += 1;
+                    } else {
+                        self.going_backward = true;
+                        self.current_frame -= 1;
+                    }
+                }
+                Mode::Once => {
+                    if self.current_frame < animation.frames.len() - 1 {
+                        self.current_frame += 1;
+                    } else {
+                        self.reset();
+                        return true;
+                    }
+                }
             }
 
             self.elapsed_in_frame -= frame.duration;
@@ -177,6 +203,7 @@ mod tests {
             SpriteSheetAnimationState {
                 current_frame: 1,
                 elapsed_in_frame: Duration::from_secs(1),
+                going_backward: false,
             }
         }
 
@@ -278,23 +305,29 @@ mod tests {
         }
     }
 
-    mod repeat {
+    mod repeat_from {
         use crate::Frame;
 
         use super::*;
 
         #[fixture]
-        fn mode() -> AnimationMode {
-            AnimationMode::Repeat
+        fn mode() -> Mode {
+            Mode::RepeatFrom(2)
         }
 
         mod on_last_frame {
             use super::*;
 
             #[fixture]
-            fn animation(frame_duration: Duration, mode: AnimationMode) -> SpriteSheetAnimation {
+            fn animation(frame_duration: Duration, mode: Mode) -> SpriteSheetAnimation {
                 SpriteSheetAnimation {
-                    frames: vec![Frame::new(0, frame_duration), Frame::new(1, frame_duration)],
+                    frames: vec![
+                        Frame::new(0, frame_duration),
+                        Frame::new(1, frame_duration),
+                        Frame::new(2, frame_duration),
+                        Frame::new(3, frame_duration),
+                        Frame::new(4, frame_duration),
+                    ],
                     mode,
                 }
             }
@@ -302,20 +335,21 @@ mod tests {
             #[fixture]
             fn state() -> SpriteSheetAnimationState {
                 SpriteSheetAnimationState {
-                    current_frame: 1,
+                    current_frame: 4,
                     elapsed_in_frame: Duration::from_nanos(0),
+                    going_backward: false,
                 }
             }
 
             #[rstest]
-            fn returns_to_first_frame(
+            fn returns_to_loop_frame(
                 mut state: SpriteSheetAnimationState,
                 mut sprite_at_second_frame: TextureAtlasSprite,
                 animation: SpriteSheetAnimation,
                 frame_duration: Duration,
             ) {
                 state.update(&mut sprite_at_second_frame, &animation, frame_duration);
-                assert_eq!(sprite_at_second_frame.index, 0);
+                assert_eq!(sprite_at_second_frame.index, 2);
             }
 
             #[rstest]
@@ -333,9 +367,14 @@ mod tests {
             use super::*;
 
             #[fixture]
-            fn animation(frame_duration: Duration, mode: AnimationMode) -> SpriteSheetAnimation {
+            fn animation(frame_duration: Duration, mode: Mode) -> SpriteSheetAnimation {
                 SpriteSheetAnimation {
-                    frames: vec![Frame::new(0, frame_duration), Frame::new(1, frame_duration)],
+                    frames: vec![
+                        Frame::new(0, frame_duration),
+                        Frame::new(1, frame_duration),
+                        Frame::new(2, frame_duration),
+                        Frame::new(3, frame_duration),
+                    ],
                     mode,
                 }
             }
@@ -343,8 +382,9 @@ mod tests {
             #[fixture]
             fn state() -> SpriteSheetAnimationState {
                 SpriteSheetAnimationState {
-                    current_frame: 2,
+                    current_frame: 4,
                     elapsed_in_frame: Duration::from_nanos(0),
+                    going_backward: false,
                 }
             }
 
@@ -356,7 +396,7 @@ mod tests {
                 frame_duration: Duration,
             ) {
                 state.update(&mut sprite_at_second_frame, &animation, frame_duration);
-                assert_eq!(sprite_at_second_frame.index, 0);
+                assert_eq!(sprite_at_second_frame.index, 2);
             }
 
             #[rstest]
@@ -375,8 +415,8 @@ mod tests {
         use super::*;
 
         #[fixture]
-        fn mode() -> AnimationMode {
-            AnimationMode::Once
+        fn mode() -> Mode {
+            Mode::Once
         }
 
         mod on_last_frame {
@@ -385,7 +425,7 @@ mod tests {
             use super::*;
 
             #[fixture]
-            fn animation(frame_duration: Duration, mode: AnimationMode) -> SpriteSheetAnimation {
+            fn animation(frame_duration: Duration, mode: Mode) -> SpriteSheetAnimation {
                 SpriteSheetAnimation {
                     frames: vec![Frame::new(0, frame_duration), Frame::new(1, frame_duration)],
                     mode,
@@ -397,6 +437,7 @@ mod tests {
                 SpriteSheetAnimationState {
                     current_frame: 1,
                     elapsed_in_frame: Duration::from_nanos(500),
+                    going_backward: false,
                 }
             }
 
